@@ -2,6 +2,8 @@ from forge_sdk.protos.protos.account_migrate_pb2 import *
 from forge_sdk.protos.protos.account_migrate_pb2_grpc import *
 from forge_sdk.protos.protos.acquire_asset_pb2 import *
 from forge_sdk.protos.protos.acquire_asset_pb2_grpc import *
+from forge_sdk.protos.protos.approve_tether_pb2 import *
+from forge_sdk.protos.protos.approve_tether_pb2_grpc import *
 from forge_sdk.protos.protos.consume_asset_pb2 import *
 from forge_sdk.protos.protos.consume_asset_pb2_grpc import *
 from forge_sdk.protos.protos.create_asset_pb2 import *
@@ -12,12 +14,18 @@ from forge_sdk.protos.protos.declare_file_pb2 import *
 from forge_sdk.protos.protos.declare_file_pb2_grpc import *
 from forge_sdk.protos.protos.declare_pb2 import *
 from forge_sdk.protos.protos.declare_pb2_grpc import *
+from forge_sdk.protos.protos.deposit_tether_pb2 import *
+from forge_sdk.protos.protos.deposit_tether_pb2_grpc import *
 from forge_sdk.protos.protos.enum_pb2 import *
 from forge_sdk.protos.protos.enum_pb2_grpc import *
 from forge_sdk.protos.protos.exchange_pb2 import *
 from forge_sdk.protos.protos.exchange_pb2_grpc import *
+from forge_sdk.protos.protos.exchange_tether_pb2 import *
+from forge_sdk.protos.protos.exchange_tether_pb2_grpc import *
 from forge_sdk.protos.protos.poke_pb2 import *
 from forge_sdk.protos.protos.poke_pb2_grpc import *
+from forge_sdk.protos.protos.revoke_tether_pb2 import *
+from forge_sdk.protos.protos.revoke_tether_pb2_grpc import *
 from forge_sdk.protos.protos.rpc_pb2 import *
 from forge_sdk.protos.protos.rpc_pb2_grpc import *
 from forge_sdk.protos.protos.service_pb2 import *
@@ -40,3 +48,5 @@ from forge_sdk.protos.protos.upgrade_node_pb2 import *
 from forge_sdk.protos.protos.upgrade_node_pb2_grpc import *
 from forge_sdk.protos.protos.vendor_pb2 import *
 from forge_sdk.protos.protos.vendor_pb2_grpc import *
+from forge_sdk.protos.protos.withdraw_tether_pb2 import *
+from forge_sdk.protos.protos.withdraw_tether_pb2_grpc import *
