@@ -1,8 +1,11 @@
 import json
 import logging
+import random
 
 import pystache
+from google.protobuf.any_pb2 import Any
 
+from forge_sdk import did
 from forge_sdk import utils
 from forge_sdk.protos import protos
 from forge_sdk.rpc.forge_rpc.chain import ForgeChainRpc
@@ -14,6 +17,8 @@ from forge_sdk.rpc.forge_rpc.wallet import ForgeWalletRpc
 
 logger = logging.getLogger('rpc')
 
+RAND_NONCE = random.randint(1, 10000)
+
 
 class ForgeRpc:
     def __init__(self, channel):
@@ -44,11 +49,16 @@ class ForgeRpc:
         if address:
             accounts = self.state_rpc.get_account_state({'address': address})
             account = next(accounts)
-            if utils.is_proto_empty(account):
-                return None
-            else:
+            if not utils.is_proto_empty(account):
                 return account.state
 
+    def get_single_tether_state(self, address):
+        if address:
+            tethers = self.state_rpc.get_tether_state({'address': address})
+            tether = next(tethers)
+            if not utils.is_proto_empty(tether):
+                return tether.state
+
     def get_single_tx_info(self, hash):
         """
         GRPC call to get transaction state of a single hash
@@ -63,9 +73,7 @@ class ForgeRpc:
         if hash:
             infos = self.chain_rpc.get_tx(hash)
             info = next(infos)
-            if utils.is_proto_empty(info):
-                return None
-            else:
+            if not utils.is_proto_empty(info):
                 return info.info
 
     def get_single_asset_state(self, address):
@@ -85,7 +93,8 @@ class ForgeRpc:
             if not utils.is_proto_empty(asset):
                 return asset.state
 
-    def build_tx(self, itx, wallet, token=None, nonce=1, chain_id=None):
+    def build_signed_tx(self, itx, wallet, token=None, nonce=RAND_NONCE,
+                        type_url=None):
         """
         Build a transaction for user. If wallet has secret key, use the
         provided
@@ -104,10 +113,19 @@ class ForgeRpc:
             :obj:`Transaction`
 
         """
-        if self._is_sk_included(wallet) and not token:
-            return self._build_tx(itx=itx, wallet=wallet, nonce=nonce, chain_id=chain_id)
+
+        encoded_itx = utils.encode_to_any(type_url, itx) if (
+            type_url and not isinstance(itx, Any)) else itx
+
+        if utils.is_sk_included(wallet) and not token:
+            return utils.build_signed_tx_local(itx=encoded_itx,
+                                               wallet=wallet,
+                                               nonce=nonce,
+                                               chain_id=self.chain_id)
         else:
-            tx_response = self.chain_rpc.create_tx(itx, wallet.address, wallet,
+            tx_response = self.chain_rpc.create_tx(encoded_itx,
+                                                   wallet.address,
+                                                   wallet,
                                                    token,
                                                    nonce)
             if tx_response.code != 0:
@@ -134,8 +152,8 @@ class ForgeRpc:
             :obj:`Transaction`
 
         """
-        if self._is_sk_included(wallet) and not token:
-            return self._build_multisig_tx(tx, wallet, data)
+        if utils.is_sk_included(wallet) and not token:
+            return utils.build_multisig_tx_local(tx, wallet, data)
         else:
             return self.chain_rpc.multisig(tx, wallet, token, data)
 
@@ -184,7 +202,7 @@ class ForgeRpc:
             logger.error(f'tx: {tx_hash} failed with code {tx_state.code}')
             return False
 
-    def create_asset(self, type_url, asset, wallet, token=None, chain_id=None, **kwargs):
+    def create_asset(self, type_url, asset, wallet, token=None, **kwargs):
         """
         GRPC call to create asset
         Create asset on the chain. If asset address is not provided in kwargs,
@@ -209,10 +227,13 @@ class ForgeRpc:
             b'sample_asset', user.wallet, user.token)
         """
 
-        itx = utils.build_create_asset_itx(type_url, asset, **kwargs)
+        itx = utils.build_create_asset_itx(type_url=type_url, asset=asset,
+                                           encoded=False, **kwargs)
 
-        tx = self.build_tx(itx=utils.encode_to_any('fg:t:create_asset', itx),
-                           wallet=wallet, token=token, chain_id=chain_id)
+        tx = self.build_signed_tx(
+            itx=utils.encode_to_any('fg:t:create_asset', itx),
+            wallet=wallet,
+            token=token)
 
         res = self.chain_rpc.send_tx(tx)
 
@@ -251,12 +272,13 @@ class ForgeRpc:
                 data=encoded_asset,
             ),
         )
-        tx = self.chain_rpc.create_tx(
-            update_asset_itx, wallet.address, wallet, token,
+        tx = self.build_signed_tx(
+            itx=update_asset_itx,
+            wallet=wallet, token=token,
         )
         return self.chain_rpc.send_tx(tx.tx)
 
-    def prepare_exchange(self, exchange_tx, wallet, token=None, chain_id=None):
+    def prepare_exchange(self, exchange_tx, wallet, token=None):
         """
         Add sender's signature to  exchange transaction
 
@@ -272,8 +294,9 @@ class ForgeRpc:
 
         """
         type_url = 'fg:t:exchange'
-        tx = self.build_tx(itx=utils.encode_to_any(type_url, exchange_tx),
-                           wallet=wallet, token=token, chain_id=chain_id)
+        tx = self.build_signed_tx(
+            itx=utils.encode_to_any(type_url, exchange_tx),
+            wallet=wallet, token=token)
         return tx
 
     def finalize_exchange(self, tx, wallet, token=None, data=None):
@@ -292,7 +315,7 @@ class ForgeRpc:
         """
         return self.build_multisig_tx(tx, wallet, token, data)
 
-    def declare(self, declare_tx, wallet, token=None):
+    def declare(self, moniker, wallet, token=None, issuer=None, data=None):
         """
         Send DeclareTx
 
@@ -311,7 +334,11 @@ class ForgeRpc:
             >>> res = declare(declare_tx, user.wallet)
 
         """
-        return self.send_itx(type_url='fg:t:declare', tx=declare_tx,
+        itx = protos.DeclareTx(moniker=moniker,
+                               issuer=issuer,
+                               data=data)
+        return self.send_itx(type_url='fg:t:declare',
+                             tx=itx,
                              wallet=wallet,
                              token=token)
 
@@ -357,7 +384,7 @@ class ForgeRpc:
 
         """
         type_url = 'fg:t:consume_asset'
-        tx = self.build_tx(utils.encode_to_any(
+        tx = self.build_signed_tx(utils.encode_to_any(
             type_url, consume_asset_tx), wallet, token)
         return tx
 
@@ -380,7 +407,7 @@ class ForgeRpc:
         return self.build_multisig_tx(tx, wallet, token, data)
 
     def create_asset_factory(self, moniker, asset, wallet, token=None,
-                             data=None, chain_id=None):
+                             data=None):
         """
         Create Asset Factory
 
@@ -399,8 +426,7 @@ class ForgeRpc:
                                  asset=asset,
                                  wallet=wallet,
                                  token=token,
-                                 data=data,
-                                 chain_id=chain_id)
+                                 data=data)
 
     def is_template_match_asset(self, template, asset):
         rendered = json.loads(pystache.render(template))
@@ -480,7 +506,8 @@ class ForgeRpc:
                                                                            spec_datas,
                                                                            type_url,
                                                                            proto_lib,
-                                                                           data)
+                                                                           data,
+                                                                           )
 
         return self.send_itx(type_url="fg:t:acquire_asset",
                              tx=acquire_asset_tx,
@@ -553,13 +580,15 @@ class ForgeRpc:
             tx_params[attribute.name] = getattr(factory_state.attributes,
                                                 attribute.name)
 
-        create_asset_itx = self.build_create_asset_tx(type_url, asset,
-                                                      **tx_params)
+        create_asset_itx = utils.build_create_asset_itx(type_url=type_url,
+                                                        asset=asset,
+                                                        encoded=False,
+                                                        **tx_params)
 
         return protos.AssetSpec(address=create_asset_itx.address,
                                 data=json.dumps(spec_data))
 
-    def send_itx(self, tx, wallet, token, type_url=None, nonce=1):
+    def send_itx(self, tx, wallet, token=None, type_url=None, nonce=RAND_NONCE):
         """
         GRPC call to send inner transaction
 
@@ -574,29 +603,95 @@ class ForgeRpc:
             :obj:`ResponseSendTx`
 
         """
-        encoded_itx = tx if not type_url else utils.encode_to_any(type_url, tx)
-        tx = self.build_tx(
-            itx=encoded_itx,
+        tx = self.build_signed_tx(
+            itx=tx,
             wallet=wallet,
             token=token,
-            nonce=nonce
+            nonce=nonce,
+            type_url=type_url,
         )
         return self.chain_rpc.send_tx(tx)
 
-    def _is_sk_included(self, wallet):
-        return wallet.sk and not wallet.sk == b''
+    def stake_for_node(self, to, value, wallet, token=None, message=None):
+        stake_address = did.get_stake_address(wallet.address, to)
+        stake_itx = protos.StakeTx(
+            to=to,
+            value=utils.int_to_bigsint(value),
+            message=message,
+            address=stake_address,
+            data=utils.encode_to_any('fg:x:stake_node',
+                                     protos.StakeForNode())
+        )
+        tx = self.build_signed_tx(type_url='fg:t:stake',
+                                  itx=stake_itx,
+                                  wallet=wallet,
+                                  token=token)
+        return self.chain_rpc.send_tx(tx)
 
-    def _build_tx(self, itx, wallet, nonce, chain_id):
-        tx = utils.build_unsigned_tx(
-            itx=itx, wallet=wallet, nonce=nonce, chain_id=chain_id)
-        tx.signature = utils.sign_tx(wallet, tx)
-        return tx
+    def prepare_deposit_tether_tx(self, wallet, token=None, **kwargs):
+        # signed by buyer
+        itx = protos.DepositTetherTx(
+            value=utils.int_to_biguint(kwargs.get('value')),
+            commission=utils.int_to_biguint(kwargs.get('commission')),
+            charge=utils.int_to_biguint(kwargs.get('charge')),
+            target=kwargs.get('target'),
+            withdrawer=kwargs.get('withdrawer'),
+            locktime=kwargs.get('locktime')
+        )
+        return self.build_signed_tx(
+            itx=utils.encode_to_any('fg:t:deposit_tether', itx),
+            wallet=wallet,
+            token=token)
 
-    def _build_multisig_tx(self, tx, wallet, data):
-        utils.add_multisigs(
-            tx, [utils.create_multisig(wallet=wallet, data=data)])
+    def finalize_deposit_tether_tx(self, tx, wallet, token=None, data=None):
+        return self.build_multisig_tx(tx=tx, wallet=wallet, token=token,
+                                      data=data)
 
-        new_multisigs = [utils.create_multisig(
-            wallet=wallet, tx=tx, data=data)]
-        utils.add_multisigs(tx, new_multisigs)
-        return tx
+    def prepare_exchange_tether(self, wallet, sender, receiver, expired_at,
+                                data=None,
+                                token=None):
+        itx = protos.ExchangeTetherTx(sender=sender,
+                                      receiver=receiver,
+                                      expired_at=expired_at,
+                                      data=data)
+        return self.build_signed_tx(itx=itx, type_url='fg:t:exchange_tether',
+                                    wallet=wallet,
+                                    token=token)
+
+    def finalize_exchange_tether_tx(self, tx, wallet, token=None, data=None):
+        return self.build_multisig_tx(tx=tx, wallet=wallet, token=token,
+                                      data=data)
+
+    def revoke_tether(self, wallet, tether, data=None, token=None,
+                      chain_id=None):
+        itx = protos.RevokeTetherTx(tether=tether, data=data)
+
+        return self.send_itx(tx=itx, type_url='fg:t:revoke_tether',
+                             wallet=wallet,
+                             token=token)
+
+    def approve_tether(self, wallet, withdraw, data=None, token=None,
+                       chain_id=None):
+        itx = protos.ApproveTetherTx(withdraw=withdraw, data=data)
+
+        return self.send_itx(tx=itx, type_url='fg:t:approve_tether',
+                             wallet=wallet,
+                             token=token)
+
+    def withdraw_tether(self, wallet, token=None, **kwargs):
+        params = {
+            'from': kwargs.get('from'),
+            'nonce': kwargs.get('nonce'),
+            'chain_id': kwargs.get('chain_id'),
+            'pk': kwargs.get('pk'),
+            'signature': kwargs.get('signature'),
+            'signatures': kwargs.get('signatures'),
+            'sender': kwargs.get('sender'),
+            'receiver': kwargs.get('receiver'),
+            'expired_at': kwargs.get('expired_at'),
+            'data': kwargs.get('data'),
+        }
+        itx = protos.WithdrawTetherTx(**params)
+        return self.send_itx(tx=itx, type_url='fg:t:withdraw_tether',
+                             wallet=wallet,
+                             token=token)
